<?php

namespace Drupal\themr_components;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\Entity\EntityViewDisplay;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\themr_components\PluginManager\ThemrFieldManager;

/**
 * Component Editor - helper functions to edit.
 */
class ComponentEditor {

  /**
   * The entitytype manager.
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The config manager.
   */
  protected ConfigFactoryInterface $config;

  /**
   * The generators.
   */
  protected array $generators = [];

  /**
   * Mapping of children.
   */
  protected array $children = [];

  /**
   * Constructs a new component.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity manager.
   * @param \Drupal\themr_components\PluginManager\ThemrFieldManager $themrFieldManager
   *   The Themr field manager.
   * @param \
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, ThemrFieldManager $themrFieldManager, ConfigFactoryInterface $config) {
    $this->entityTypeManager = $entityTypeManager;
    foreach ($themrFieldManager->getDefinitions() as $definition) {
      $this->generators[$definition['id']] = $themrFieldManager->createInstance($definition['id']);
    }
    $this->config = $config;
  }

  /**
   * Resave a field.
   */
  public function saveFieldValue($componentName, $name, $saveValue, $attributeKey, $removeIfEmpty = TRUE) {
    $fields = $this->config->get('themr_components.component.' . $componentName)->get('fields');
    foreach ($fields as $key => $field) {
      if ($key == $name) {
        if (!empty($saveValue)) {
          $fields[$key][$attributeKey] = $saveValue;
        }
        else if (isset($fields[$key][$attributeKey]) && $removeIfEmpty) {
          unset($fields[$key][$attributeKey]);
        }
      }
    }
    $configEditable = $this->config->getEditable('themr_components.component.' . $componentName)->set('fields', $fields);
    $configEditable->save();
  }

  /**
   * Resave any container.
   */
  public function saveConfigValue($componentName, $name, $saveValue, $attributeKey, $removeIfEmpty = TRUE) {
    $config = $this->config->get('themr_components.component.' . $componentName)->get('config');
    $newConfig = $this->setChange($componentName, $name, $saveValue, $attributeKey, $config, $removeIfEmpty);
    $configEditable = $this->config->getEditable('themr_components.component.' . $componentName)->set('config', $newConfig);
    $configEditable->save();

    // Save field as well.
    $this->saveFieldValue($componentName, $name, $saveValue, $attributeKey, $removeIfEmpty);
  }

  /**
   * Set change.
   */
  public function setChange($componentName, $name, $saveValue, $attributeKey, $config, $removeIfEmpty = TRUE) {
    foreach ($config as $key => $part) {
      if (isset($part['contains'])) {
        $config[$key]['contains'] = $this->setChange($componentName, $name, $saveValue, $attributeKey, $part['contains'], $removeIfEmpty);
      }
      else if ($part['key'] == $name) {
        if (!empty($saveValue)) {
          $config[$key][$attributeKey] = $saveValue;
        }
        else if (isset($config[$key][$attributeKey]) && $removeIfEmpty) {
          unset($config[$key][$attributeKey]);
        }
      }
    }
    return $config;
  }

}
