<?php

/**
 * @file
 * Contains \Drupal\themr_components\Form\ComponentGenerator.
 */

namespace Drupal\themr_components\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class AnimationModificationForm extends FormBase implements ContainerInjectionInterface {

  /**
   * Entity type manager.
   */
  protected ConfigFactoryInterface $configMutable;

  /**
   * On scroll animation.
   */
  protected array $aosAnimation = [
    '' => '-- None --',
    'fade-up' => 'Fade Up',
    'fade-down' => 'Fade Down',
    'fade-right' => 'Fade Right',
    'fade-left' => 'Fade Left',
    'fade-up-right' => 'Fade Up Right',
    'fade-up-left' => 'Fade Up Left',
    'fade-down-right' => 'Fade Down Right',
    'fade-down-left' => 'Fade Down Left',
    'flip-left' => 'Flip Left',
    'flip-right' => 'Flip Right',
    'flip-up' => 'Flip Up',
    'flip-down' => 'Flip Down',
    'zoom-in' => 'Zoom In',
    'zoom-in-up' => 'Zoom In Up',
    'zoom-in-down' => 'Zoom In Down',
    'zoom-in-left' => 'Zoom In Left',
    'zoom-in-right' => 'Zoom In Right',
    'zoom-out' => 'Zoom Out',
    'zoom-out-up' => 'Zoom Out Up',
    'zoom-out-down' => 'Zoom Out Down',
    'zoom-out-right' => 'Zoom Out Right',
    'zoom-out-left' => 'Zoom Out Left',
  ];

  /**
   * Constructor.
   */
  public function __construct(
    ConfigFactoryInterface $configMutable,
  ) {
    $this->configMutable = $configMutable;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
    );
  }
  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'themr_animation_modification_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $component = NULL, $element_type = NULL, $name = NULL) {
    $form['component'] = [
      '#type' => 'value',
      '#value' => $component,
    ];

    $form['element_type'] = [
      '#type' => 'value',
      '#value' => $element_type,
    ];

    $form['name'] = [
      '#type' => 'value',
      '#value' => $name,
    ];

    $form['animation_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Select on-scroll animation'),
      '#description' => $this->t('On-scroll animation are animations that are triggered when the element comes into the viewport.'),
      '#options' => $this->aosAnimation,
      '#default_value' => $this->loadAnimationValue($component, $element_type, $name),
      '#ajax' => [
        'callback' => [$this, 'ajaxAnimation'],
        'event' => 'change',
      ],
      '#executes_submit_callback' => TRUE,
    ];

    $form['test'] = [
      '#type' => 'button',
      '#value' => $this->t('Test animation'),
      '#attributes' => [
        'id' => 'test-animate',
      ],
    ];

    $form['test-direct'] = [
      '#type' => 'button',
      '#value' => $this->t('Test animation Direct'),
      '#attributes' => [
        'id' => 'test-animate-direct',
        'style' => 'display: none;',
      ],
    ];

    return $form;
  }


  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $formState) {
   // Save the animation and then trigger refresh.
   $this->saveAnimationValue($formState->getValue('component'), $formState->getValue('element_type'), $formState->getValue('name'), $formState->getValue('animation_type'));
  }

  /**
   * {@inheritdoc}
   */
  public function ajaxAnimation(array &$form, FormStateInterface $formState) {
    $response = new AjaxResponse();
    $response->addCommand(new InvokeCommand('#edit-regenerate', 'click'));
    $response->addCommand(new InvokeCommand('#test-animate-direct', 'click'));
    return $response;
  }

  /**
   * Load animation.
   */
  public function loadAnimationValue($componentName, $elementType, $name) {
    $fields = $this->config('themr_components.component.' . $componentName)->get('fields');
    foreach ($fields as $key => $field) {
      if ($key == $name) {
        if (isset($fields[$key]['attributes']['data-aos'])) {
          return $fields[$key]['attributes']['data-aos'];
        }
      }
    }
    return "";
  }

  /**
   * Resave a field/wrapper/animation.
   */
  public function saveAnimationValue($componentName, $elementType, $name, $value) {
    $fields = $this->configMutable->get('themr_components.component.' . $componentName)->get('fields');
    foreach ($fields as $key => $field) {
      if ($key == $name) {
        if ($value) {
          $fields[$key]['attributes']['data-aos'] = $value;
        }
        else if (isset($fields[$key]['attributes']['data-aos'])) {
          unset($fields[$key]['attributes']['data-aos']);
        }
      }
    }
    $config = $this->configMutable->getEditable('themr_components.component.' . $componentName)->set('fields', $fields);
    $config->save();
  }

}
