<?php

/**
 * @file
 * Contains \Drupal\themr_components\Form\ComponentGenerator.
 */

namespace Drupal\themr_components\Form;

use Drupal\Core\Ajax\AddCssCommand;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CssCommand;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\themr_components\ComponentEditor;
use Symfony\Component\DependencyInjection\ContainerInterface;

class TestValueModificationForm extends FormBase implements ContainerInjectionInterface {

  /**
   * Entity type manager.
   */
  protected ConfigFactoryInterface $configMutable;

  /**
   * Component editor.
   */
  protected ComponentEditor $editor;

  /**
   * Constructor.
   */
  public function __construct(
    ConfigFactoryInterface $configMutable,
    ComponentEditor $editor,
  ) {
    $this->configMutable = $configMutable;
    $this->editor = $editor;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('themr_components.editor'),
    );
  }
  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'themr_css_modification_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $component = NULL, $element_type = NULL, $name = NULL) {
    $css = $this->getValue($component, $element_type, $name);

    $form['component'] = [
      '#type' => 'value',
      '#value' => $component,
    ];

    $form['element_type'] = [
      '#type' => 'value',
      '#value' => $element_type,
    ];

    $form['name'] = [
      '#type' => 'value',
      '#value' => $name,
    ];

    $form['value'] = [
      '#title' => $this->t('Value'),
      '#type' => 'textarea',
      '#default_value' => $css,
    ];

    $form['test-direct'] = [
      '#type' => 'button',
      '#value' => $this->t('Change Value'),
      '#attributes' => [
        //'style' => 'display: none;',
      ],
      '#ajax' => [
        'callback' => [$this, 'ajaxCss'],
        'event' => 'click',
      ],
      '#executes_submit_callback' => TRUE,
    ];

    return $form;
  }


  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $formState) {
   // Save the animation and then trigger refresh.
   $this->saveCssValue($formState->getValue('component'), $formState->getValue('element_type'), $formState->getValue('name'), $formState->getValue('css'));
  }

  /**
   * {@inheritdoc}
   */
  public function ajaxCss(array &$form, FormStateInterface $formState) {
    $response = new AjaxResponse();
    return $response;
  }

  /**
   * Load css.
   */
  public function getValue($componentName, $elementType, $name) {
    $fields = $this->config('themr_components.component.' . $componentName)->get('fields');
    foreach ($fields as $key => $field) {
      if ($key == $name) {
        if (isset($fields[$key]['field-example-value'])) {
          return $fields[$key]['field-example-value'];
        }
      }
    }
    return "";
  }

  /**
   * Resave a field/wrapper/animation.
   */
  public function saveCssValue($componentName, $elementType, $name, $value) {
    $saveValue = [];
    foreach (explode("\n", $value) as $row) {
      if (strpos($row, ':') !== FALSE) {
        $row = str_replace(';', '', $row);
        $parts = explode(':', $row);
        if (count($parts) == 2) {
          $saveValue[trim($parts[0])] = trim($parts[1]);
        }
      }
    }
    $this->editor->saveConfigValue($componentName, $name, $saveValue, 'css');
  }

}
