<?php

/**
 * @file
 * Contains \Drupal\themr_components\Form\ComponentGenerator.
 */

namespace Drupal\themr_components\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class VisualAttributeModificationForm extends FormBase implements ContainerInjectionInterface {

  /**
   * Entity type manager.
   */
  protected ConfigFactoryInterface $configMutable;

  /**
   * Constructor.
   */
  public function __construct(
    ConfigFactoryInterface $configMutable,
  ) {
    $this->configMutable = $configMutable;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
    );
  }
  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'themr_visual_attributes_modification_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $component = NULL, $element_type = NULL, $name = NULL) {
    $form['component'] = [
      '#type' => 'value',
      '#value' => $component,
    ];

    $form['element_type'] = [
      '#type' => 'value',
      '#value' => $element_type,
    ];

    $form['name'] = [
      '#type' => 'value',
      '#value' => $name,
    ];

    $form['background-color'] = [
      '#title' => $this->t('Background Color'),
      '#type' => 'color',
    ];

    return $form;
  }


  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $formState) {
   // Save the animation and then trigger refresh.
   $this->saveAnimationValue($formState->getValue('component'), $formState->getValue('element_type'), $formState->getValue('name'), $formState->getValue('animation_type'));
  }

  /**
   * {@inheritdoc}
   */
  public function ajaxAnimation(array &$form, FormStateInterface $formState) {
    $response = new AjaxResponse();
    $response->addCommand(new InvokeCommand('#edit-regenerate', 'click'));
    $response->addCommand(new InvokeCommand('#test-animate-direct', 'click'));
    return $response;
  }

  /**
   * Load animation.
   */
  public function loadAnimationValue($componentName, $elementType, $name) {
    $fields = $this->config('themr_components.component.' . $componentName)->get('fields');
    foreach ($fields as $key => $field) {
      if (\Drupal::service('themr_components.generator')->getDataName($field['field-name']) == $name) {
        if (isset($fields[$key]['attributes']['data-aos'])) {
          return $fields[$key]['attributes']['data-aos'];
        }
      }
    }
    return "";
  }

  /**
   * Resave a field/wrapper/animation.
   */
  public function saveAnimationValue($componentName, $elementType, $name, $value) {
    $fields = $this->configMutable->get('themr_components.component.' . $componentName)->get('fields');
    foreach ($fields as $key => $field) {
      if (\Drupal::service('themr_components.generator')->getDataName($field['field-name']) == $name) {
        if ($value) {
          $fields[$key]['attributes']['data-aos'] = $value;
        }
        else if (isset($fields[$key]['attributes']['data-aos'])) {
          unset($fields[$key]['attributes']['data-aos']);
        }
      }
    }
    $config = $this->configMutable->getEditable('themr_components.component.' . $componentName)->set('fields', $fields);
    $config->save();
  }

}
