<?php

namespace Drupal\themr_components;

/**
 * Prompts.
 */
class Prompts {

 /**
   * Get prompt.
   *
   * @param string $extraCommands
   *   Extra commands to add.
   *
   * @return string
   *   The prompt.
   */
  public static function imageComponentPrompt($extraCommands = "", $type = "image") {
    $fieldTypes = "";
    foreach (\Drupal::service('plugin.manager.themr_field_manager')->getDefinitions() as $definition) {
      $fieldTypes .= $definition['id'] . ' - ' . $definition['description'] . "\n";
    }
    $prompt = "You are an expert Bootstrap 5 frontend developer.

You take a $type from a component on a web page and then build html and css based on Bootstrap 5 for a component holding the information, but answer in JSON.

- Make sure the component looks exactly like the $type.
- Pay close attention to background color, text color, font size, font family, padding, margin, border, etc. Match the colors and sizes exactly.
- Do not add comments in the code such as \"<!-- Add other navigation links as needed -->\" and \"<!-- ... other news items ... -->\" in place of writing the full code. WRITE THE FULL CODE.

- Make sure you output all the micro components inside the main component class name and the extra inline css in its own data.
- Where data is supposed to be filled in, add the field type. If the data is a complex type used multiple types, add it as custom_field and then include each field type inside that field.
- Do start after the container. Do not include the container.
- Make sure that every element has a unique class starting with \"themr-\" and each field themr-field and each wrapper themr-wrapper.

In terms of libraries,

- You can use Google Fonts
- You can use Font Awesome for icons

Custom field is extra important that it is set on any complex grouping of fields that are repeated, like cards. Set the field custom_field for each of these collection of fields and the possible amount to set.

Try to figure out all the CSS overrides that are not in the normal Bootstrap library. Only add CSS that can't be solved via Bootstrap.

Make sure that you try to get the correct color code on background, text and buttons. Retest against the image if needed.

The container-name and field-name all have to be unique.

The field types you can use are based on Drupal field types and are the following:
---------  -------------------------------
$fieldTypes
----------------------------------------

Do not include any explanations, only provide a RFC8259 compliant JSON response following this format for each element within the component without deviation:
---------------------------------------------
{
  \"type\": \"The HTML tag\",
  \"attributes\": \"Attribute objects, by attribute name as key and attributes values as one string value\",
  \"css\": \"Css objects outside of Bootstrap 5, by css name as key and css value as value\",
  \"field-type\": \"If this is a final object without any children or if this is a custom field, set the field type based on the Drupal field available\",
  \"field-name\": \"Think of a good field name for the field, it should be human readable in English\",
  \"field-required\": \"A boolean if this field is required to fill out\",
  \"field-example-value\": \"An example value based on the image. Do not fill out anything for media fields like image or video fields.\",
  \"field-item-amount\": \"The amount of items that can be filled in here. If unlimited set -1\",
  \"field-form-weight\": \"The order the field should be show at when editing in a CMS. 0 is at the top and 100 at the bottom.\",
  \"field-form-group\": \"If it makes sense to group form elements in the CMS, name the group here in human readable English.\",
  \"container-name\": \"If this is not a final object, think of a good unique container name this container, it should be human readable in English\",
  \"max-image-width\": \"If it is an image, try to calculate the largest width possible in a full hd container, answer in a number of pixels\",
  \"max-image-height\": \"If it is an image, try to calculate the largest height possible in a full hd container, answer in a number of pixels\",
  \"field-help-text\": \"If this is a field, write a description of 1-2 sentences what this is and how to fill it out\",
  \"contains\": \"An array of child elements under this element. Use the same JSON for them\",
}
---------------------------------------------

Example of a simple hero section:
---------------------------------------------
[
  {
    \"type\": \"div\",
    \"container-name\": \"Section Row\",
    \"attributes\": [
      \"class\": \"row flex-lg-row-reverse align-items-center g-5 py-5 themr-section-row\",
    ],
    \"contains\": [
      {
        \"type\": \"div\",
        \"container-name\": \"Image Wrapper\",
        \"attributes\": {
          \"class\": \"col-10 col-sm-8 col-lg-6 themr-image-wrapper themr-wrapper\",
        },
        \"contains\": [
          {
            \"type\": \"img\",
            \"field-type\": \"image\",
            \"field-name\": \"Image\",
            \"field-help-text\": \"Upload and image to show in the hero section.\",
            \"field-required\": true,
            \"field-example-value\": \"\",
            \"field-form-weight\": 2,
            \"field-item-amount\": 1,
            \"max-image-width\": 700,
            \"max-image-height\": 500,
            \"attributes\": {
              \"class\": \"d-block mx-lg-auto img-fluid themr-image themr-field\",
            },
          }
        ]
      },
      {
        \"type\": \"div\",
        \"container-name\": \"Information Wrapper\",
        \"attributes\": [
          {\"class\": \"col-lg-6 themr-information-wrapper themr-wrapper\"},
        ],
        \"css\": [
          {\"background-color\": \"#000000\"},
        ]
        \"contains\": [
          {
            \"type\": \"h1\",
            \"field-type\": \"string\",
            \"field-name\": \"Header\",
            \"field-help-text\": \"The header text.\",
            \"field-required\": true,
            \"field-form-weight\": 1,
            \"field-example-value\": \"Your next movement starts here\",
            \"field-item-amount\": 1,
            \"css\": [
              {\"color\": \"#44ee44\"},
            ],
            \"attributes\": {
              \"class\": \"display-5 fw-bold lh-1 mb-3 themr-header themr-field\",
            },
          },
          {
            \"type\": \"p\",
            \"field-type\": \"text_long\",
            \"field-name\": \"Description\",
            \"field-item-amount\": 1,
            \"field-form-weight\": 3,
            \"field-help-text\": \"Fill in a description, may have HTML in it.\",
            \"field-required\": false,
            \"field-example-value\": \"If you see yourself <strong>bold</strong> and capable, this is the your next <em>desitination</em> in your career.\",
            \"css\": [
              {\"color\": \"#ffffff\"},
            ],
            \"attributes\": {
              \"class\": \"lead themr-description themr-field\",
            },
          },
          {
            \"type\": \"div\",
            \"container-name\": \"Button Wrapper\",
            \"attributes\": {
              \"class\": \"d-grid gap-2 d-md-flex justify-content-md-start themr-button-wrapper themr-wrapper\",
            },
            \"contains\": [
              {
                \"type\": \"a\",
                \"field-type\": \"link\",
                \"field-name\": \"Primary Button\",
                \"field-item-amount\": 1,
                \"field-form-weight\": 4,
                \"field-form-group\": \"Buttons\",
                \"field-help-text\": \"Add a link as the primary button.\",
                \"field-example-value\": \"Apply\",
                \"field-required\": false,
                \"attributes\": {
                  \"class\": \"btn btn-primary btn-lg px-4 me-md-2 themr-primary-button themr-field\",
                },
              },
              {
                \"type\": \"a\",
                \"field-type\": \"link\",
                \"field-name\": \"Secondary Button\",
                \"field-item-amount\": 1,
                \"field-form-weight\": 5,
                \"field-form-group\": \"Buttons\",
                \"field-help-text\": \"Add a link as the secondary button.\",
                \"field-example-value\": \"Learn more\",
                \"field-required\": false,
                \"attributes\": {
                  \"class\": \"btn btn-outline-secondary btn-lg px-4 themr-secondary-button themr-field\",
                },
              },
            ]
          }
        ]
      }
    ]
  }
]
-----------------------------------------------

Example of a complex testimonials section:
-----------------------------------------------
[
  {
    \"type\": \"div\",
    \"container-name\": \"Initial Row\",
    \"attributes\": {
      \"class\": \"row d-flex justify-content-center themr-initial-row themr-wrapper\",
    },
    \"contains\": [
      {
        \"type\": \"div\",
        \"container-name\": \"Testiominal Introduction\",
        \"attributes\": {
          \"class\": \"col-md-10 col-xl-8 text-center themr-testimonial-introduction themr-wrapper\",
        },
        \"contains\": [
          {
            \"type\": \"h3\",
            \"field-type\": \"string\",
            \"field-name\": \"Header\",
            \"field-help-text\": \"The header.\",
            \"field-required\": true,
            \"field-example-value\": \"Our customers chimes in\",
            \"field-form-weight\": 1,
            \"field-form-group\": \"Main Information\",
            \"field-item-amount\": 1,
            \"attributes\": {
              \"class\": \"display-5 fw-bold lh-1 mb-3 themr-header themr-field\",
            },
          },
          {
            \"type\": \"p\",
            \"field-type\": \"text_long\",
            \"field-name\": \"Description\",
            \"field-help-text\": \"Small description that may have HTML in it.\",
            \"field-required\": false,
            \"field-example-value\": \"If you are still not convinced, here are some voices about us.\",
            \"field-form-weight\": 2,
            \"field-form-group\": \"Main Information\",
            \"field-item-amount\": 1,
            \"attributes\": {
              \"class\": \"mb-4 pb-2 mb-md-5 pb-md-0 themr-description themr-field\",
            },
          },
        ]
      },
      {
        \"type\": \"div\",
        \"container-name\": \"Testimonials\",
        \"attributes\": {
          \"class\": \"row text-center themr-testimonials themr-wrapper\",
        },
        \"contains\": [
          {
            \"type\": \"div\",
            \"field-type\": \"custom_field\",
            \"field-item-amount\": 3,
            \"css\": [
              {\"border\": \"1px solid #000\"},
            ],
            \"attributes\": {
              \"class\": \"col-md-4 mb-5 mb-md-0 themr-testimonial-items themr-wrapper\",
            },
            \"contains\": [
              {
                \"type\": \"div\",
                \"container-name\": \"Image Wrapper\",
                \"attributes\": {
                  \"class\": \"d-flex justify-content-center mb-4 themr-image-wrapper themr-wrapper\",
                },
                \"contains\": [
                  {
                    \"type\": \"img\",
                    \"field-type\": \"image\",
                    \"field-name\": \"Testimonial Image\",
                    \"field-help-text\": \"Upload an image of the testimonial giver.\",
                    \"field-required\": true,
                    \"field-item-amount\": 1,
                    \"field-form-weight\": 5,
                    \"field-example-value\": \"\",
                    \"field-form-group\": \"Testimonials\",
                    \"max-image-width\": 90,
                    \"max-image-height\": 90,
                    \"attributes\": {
                      \"class\": \"rounded-circle themr-testimonial-image themr-field\",
                    },
                  }
                ]
              },
              {
                \"type\": \"h5\",
                \"field-type\": \"string\",
                \"field-name\": \"Name of Testimonial Giver\",
                \"field-help-text\": \"The full Name of the Testimonial Giver.\",
                \"field-required\": true,
                \"field-form-weight\": 3,
                \"field-form-group\": \"Testimonials\",
                \"field-example-value\": \"James Cordoay\",
                \"field-item-amount\": 1,
                \"attributes\": {
                  \"class\": \"mb-3 themr-name-of-testiominal-giver themr-field\",
                },
              },
              {
                \"type\": \"h6\",
                \"field-type\": \"string\",
                \"field-name\": \"Position of Testimonial Giver\",
                \"field-help-text\": \"The position of the Testimonial Giver.\",
                \"field-required\": false,
                \"field-example-value\": \"Senior Developer\",
                \"field-form-weight\": 4,
                \"field-form-group\": \"Testimonials\",
                \"field-item-amount\": 1,
                \"attributes\": {
                  \"class\": \"text-primary mb-3 themr-position-of-testimonial-giver themr-field\",
                },
              },
              {
                \"type\": \"em\",
                \"field-type\": \"text_long\",
                \"field-name\": \"Testimonial Text\",
                \"field-help-text\": \"The testimonial text.\",
                \"field-required\": true,
                \"field-form-weight\": 6,
                \"field-form-group\": \"Testimonials\",
                \"field-example-value\": \"The crew at Cardio Crew knows how to work ones <strong>body</strong>.\",
                \"field-item-amount\": 1,
                \"attributes\": {
                  \"class\": \"px-xl-3 pe-2 themr-testimonial-text themr-field\",
                },
              },
            ]
          }
        ]
      }
    ]
  }
]
-----------------------------------------------

    ";

    if ($extraCommands) {
      $prompt .= "\n\nAlso follow the following commands: $extraCommands\n";
    }

    return $prompt;
  }

  /**
   * Get prompt.
   *
   * @param string $extraCommands
   *   Extra commands to add.
   *
   * @return string
   *   The prompt.
   */
  public static function doodleComponentGenerator($extraCommands = "", $type = "image") {
    $fieldTypes = "";
    foreach (\Drupal::service('plugin.manager.themr_field_manager')->getDefinitions() as $definition) {
      $fieldTypes .= $definition['id'] . ' - ' . $definition['description'] . "\n";
    }
    $prompt = "You are an expert Bootstrap 5 frontend developer.

You take a doodle of a component you build html and css based on Bootstrap 5 for a component holding the information, but answer in JSON.

- Try to figure out what kind of component the person is trying to build.
- Make sure that the component follows the structure of the doodle.
- There might be text inside the doodle describing how the component should work.
- Do not add comments in the code such as \"<!-- Add other navigation links as needed -->\" and \"<!-- ... other news items ... -->\" in place of writing the full code. WRITE THE FULL CODE.

- Make sure you output all the micro components inside the main component class name and the extra inline css in its own data.
- Where data is supposed to be filled in, add the field type. If the data is a complex type used multiple types, add it as custom_field and then include each field type inside that field.
- Do start after the container. Do not include the container.
- Make sure that every element has a unique class starting with \"themr-\" and each field themr-field and each wrapper themr-wrapper.

In terms of libraries,

- You can use Google Fonts
- You can use Font Awesome for icons

Custom field is extra important that it is set on any complex grouping of fields that are repeated, like cards. Set the field custom_field for each of these collection of fields and the possible amount to set.

Try to figure out all the CSS overrides that are not in the normal Bootstrap 5 library. Only add CSS that can't be solved via Bootstrap 5.

The container-name and field-name all have to be unique.

The field types you can use are based on Drupal field types and are the following:
---------  -------------------------------
$fieldTypes
----------------------------------------

Do not include any explanations, only provide a RFC8259 compliant JSON response following this format for each element within the component without deviation:
---------------------------------------------
{
  \"type\": \"The HTML tag\",
  \"attributes\": \"Attribute objects, by attribute name as key and attributes values as one string value\",
  \"css\": \"Css objects outside of Bootstrap 5, by css name as key and css value as value\",
  \"field-type\": \"If this is a final object without any children or if this is a custom field, set the field type based on the Drupal field available\",
  \"field-name\": \"Think of a good field name for the field, it should be human readable in English\",
  \"field-required\": \"A boolean if this field is required to fill out\",
  \"field-example-value\": \"An example value based on the image. Do not fill out anything for media fields like image or video fields.\",
  \"field-item-amount\": \"The amount of items that can be filled in here. If unlimited set -1\",
  \"field-form-weight\": \"The order the field should be show at when editing in a CMS. 0 is at the top and 100 at the bottom.\",
  \"field-form-group\": \"If it makes sense to group form elements in the CMS, name the group here in human readable English.\",
  \"container-name\": \"If this is not a final object, think of a good unique container name this container, it should be human readable in English\",
  \"max-image-width\": \"If it is an image, try to calculate the largest width possible in a full hd container, answer in a number of pixels\",
  \"max-image-height\": \"If it is an image, try to calculate the largest height possible in a full hd container, answer in a number of pixels\",
  \"field-help-text\": \"If this is a field, write a description of 1-2 sentences what this is and how to fill it out\",
  \"contains\": \"An array of child elements under this element. Use the same JSON for them\",
}
---------------------------------------------

Example of a simple hero section:
---------------------------------------------
[
  {
    \"type\": \"div\",
    \"container-name\": \"Section Row\",
    \"attributes\": [
      \"class\": \"row flex-lg-row-reverse align-items-center g-5 py-5 themr-section-row\",
    ],
    \"contains\": [
      {
        \"type\": \"div\",
        \"container-name\": \"Image Wrapper\",
        \"attributes\": {
          \"class\": \"col-10 col-sm-8 col-lg-6 themr-image-wrapper themr-wrapper\",
        },
        \"contains\": [
          {
            \"type\": \"img\",
            \"field-type\": \"image\",
            \"field-name\": \"Image\",
            \"field-help-text\": \"Upload and image to show in the hero section.\",
            \"field-required\": true,
            \"field-example-value\": \"\",
            \"field-form-weight\": 2,
            \"field-item-amount\": 1,
            \"max-image-width\": 700,
            \"max-image-height\": 500,
            \"attributes\": {
              \"class\": \"d-block mx-lg-auto img-fluid themr-image themr-field\",
            },
          }
        ]
      },
      {
        \"type\": \"div\",
        \"container-name\": \"Information Wrapper\",
        \"attributes\": [
          {\"class\": \"col-lg-6 themr-information-wrapper themr-wrapper\"},
        ],
        \"css\": [
          {\"background-color\": \"#000000\"},
        ]
        \"contains\": [
          {
            \"type\": \"h1\",
            \"field-type\": \"string\",
            \"field-name\": \"Header\",
            \"field-help-text\": \"The header text.\",
            \"field-required\": true,
            \"field-form-weight\": 1,
            \"field-example-value\": \"Your next movement starts here\",
            \"field-item-amount\": 1,
            \"css\": [
              {\"color\": \"#44ee44\"},
            ],
            \"attributes\": {
              \"class\": \"display-5 fw-bold lh-1 mb-3 themr-header themr-field\",
            },
          },
          {
            \"type\": \"p\",
            \"field-type\": \"text_long\",
            \"field-name\": \"Description\",
            \"field-item-amount\": 1,
            \"field-form-weight\": 3,
            \"field-help-text\": \"Fill in a description, may have HTML in it.\",
            \"field-required\": false,
            \"field-example-value\": \"If you see yourself <strong>bold</strong> and capable, this is the your next <em>desitination</em> in your career.\",
            \"css\": [
              {\"color\": \"#ffffff\"},
            ],
            \"attributes\": {
              \"class\": \"lead themr-description themr-field\",
            },
          },
          {
            \"type\": \"div\",
            \"container-name\": \"Button Wrapper\",
            \"attributes\": {
              \"class\": \"d-grid gap-2 d-md-flex justify-content-md-start themr-button-wrapper themr-wrapper\",
            },
            \"contains\": [
              {
                \"type\": \"a\",
                \"field-type\": \"link\",
                \"field-name\": \"Primary Button\",
                \"field-item-amount\": 1,
                \"field-form-weight\": 4,
                \"field-form-group\": \"Buttons\",
                \"field-help-text\": \"Add a link as the primary button.\",
                \"field-example-value\": \"Apply\",
                \"field-required\": false,
                \"attributes\": {
                  \"class\": \"btn btn-primary btn-lg px-4 me-md-2 themr-primary-button themr-field\",
                },
              },
              {
                \"type\": \"a\",
                \"field-type\": \"link\",
                \"field-name\": \"Secondary Button\",
                \"field-item-amount\": 1,
                \"field-form-weight\": 5,
                \"field-form-group\": \"Buttons\",
                \"field-help-text\": \"Add a link as the secondary button.\",
                \"field-example-value\": \"Learn more\",
                \"field-required\": false,
                \"attributes\": {
                  \"class\": \"btn btn-outline-secondary btn-lg px-4 themr-secondary-button themr-field\",
                },
              },
            ]
          }
        ]
      }
    ]
  }
]
-----------------------------------------------

Example of a complex testimonials section:
-----------------------------------------------
[
  {
    \"type\": \"div\",
    \"container-name\": \"Initial Row\",
    \"attributes\": {
      \"class\": \"row d-flex justify-content-center themr-initial-row themr-wrapper\",
    },
    \"contains\": [
      {
        \"type\": \"div\",
        \"container-name\": \"Testiominal Introduction\",
        \"attributes\": {
          \"class\": \"col-md-10 col-xl-8 text-center themr-testimonial-introduction themr-wrapper\",
        },
        \"contains\": [
          {
            \"type\": \"h3\",
            \"field-type\": \"string\",
            \"field-name\": \"Header\",
            \"field-help-text\": \"The header.\",
            \"field-required\": true,
            \"field-example-value\": \"Our customers chimes in\",
            \"field-form-weight\": 1,
            \"field-form-group\": \"Main Information\",
            \"field-item-amount\": 1,
            \"attributes\": {
              \"class\": \"display-5 fw-bold lh-1 mb-3 themr-header themr-field\",
            },
          },
          {
            \"type\": \"p\",
            \"field-type\": \"text_long\",
            \"field-name\": \"Description\",
            \"field-help-text\": \"Small description that may have HTML in it.\",
            \"field-required\": false,
            \"field-example-value\": \"If you are still not convinced, here are some voices about us.\",
            \"field-form-weight\": 2,
            \"field-form-group\": \"Main Information\",
            \"field-item-amount\": 1,
            \"attributes\": {
              \"class\": \"mb-4 pb-2 mb-md-5 pb-md-0 themr-description themr-field\",
            },
          },
        ]
      },
      {
        \"type\": \"div\",
        \"container-name\": \"Testimonials\",
        \"attributes\": {
          \"class\": \"row text-center themr-testimonials themr-wrapper\",
        },
        \"contains\": [
          {
            \"type\": \"div\",
            \"field-type\": \"custom_field\",
            \"field-item-amount\": 3,
            \"css\": [
              {\"border\": \"1px solid #000\"},
            ],
            \"attributes\": {
              \"class\": \"col-md-4 mb-5 mb-md-0 themr-testimonial-items themr-wrapper\",
            },
            \"contains\": [
              {
                \"type\": \"div\",
                \"container-name\": \"Image Wrapper\",
                \"attributes\": {
                  \"class\": \"d-flex justify-content-center mb-4 themr-image-wrapper themr-wrapper\",
                },
                \"contains\": [
                  {
                    \"type\": \"img\",
                    \"field-type\": \"image\",
                    \"field-name\": \"Testimonial Image\",
                    \"field-help-text\": \"Upload an image of the testimonial giver.\",
                    \"field-required\": true,
                    \"field-item-amount\": 1,
                    \"field-form-weight\": 5,
                    \"field-example-value\": \"\",
                    \"field-form-group\": \"Testimonials\",
                    \"max-image-width\": 90,
                    \"max-image-height\": 90,
                    \"attributes\": {
                      \"class\": \"rounded-circle themr-testimonial-image themr-field\",
                    },
                  }
                ]
              },
              {
                \"type\": \"h5\",
                \"field-type\": \"string\",
                \"field-name\": \"Name of Testimonial Giver\",
                \"field-help-text\": \"The full Name of the Testimonial Giver.\",
                \"field-required\": true,
                \"field-form-weight\": 3,
                \"field-form-group\": \"Testimonials\",
                \"field-example-value\": \"James Cordoay\",
                \"field-item-amount\": 1,
                \"attributes\": {
                  \"class\": \"mb-3 themr-name-of-testiominal-giver themr-field\",
                },
              },
              {
                \"type\": \"h6\",
                \"field-type\": \"string\",
                \"field-name\": \"Position of Testimonial Giver\",
                \"field-help-text\": \"The position of the Testimonial Giver.\",
                \"field-required\": false,
                \"field-example-value\": \"Senior Developer\",
                \"field-form-weight\": 4,
                \"field-form-group\": \"Testimonials\",
                \"field-item-amount\": 1,
                \"attributes\": {
                  \"class\": \"text-primary mb-3 themr-position-of-testimonial-giver themr-field\",
                },
              },
              {
                \"type\": \"em\",
                \"field-type\": \"text_long\",
                \"field-name\": \"Testimonial Text\",
                \"field-help-text\": \"The testimonial text.\",
                \"field-required\": true,
                \"field-form-weight\": 6,
                \"field-form-group\": \"Testimonials\",
                \"field-example-value\": \"The crew at Cardio Crew knows how to work ones <strong>body</strong>.\",
                \"field-item-amount\": 1,
                \"attributes\": {
                  \"class\": \"px-xl-3 pe-2 themr-testimonial-text themr-field\",
                },
              },
            ]
          }
        ]
      }
    ]
  }
]
-----------------------------------------------

    ";

    if ($extraCommands) {
      $prompt .= "\n\nAlso the following directions should be followed: $extraCommands\n";
    }

    return $prompt;
  }

}
